SystemOrganization addCategory: #'Collections-Traits'!

Trait named: #TSortable
	uses: {}
	category: 'Collections-Traits'!

!TSortable methodsFor: 'sorting' stamp: 'MarcusDenker 3/5/2010 16:47'!
isSorted
	"Return true if the receiver is sorted by the given criterion.
	Optimization for isSortedBy: [:a :b | a <= b]."

	| lastElm elm |
	self isEmpty ifTrue: [^ true].
	lastElm := self first.
	2 to: self size do: 
		[:index | 
		elm := self at: index.
		lastElm <= elm ifFalse: [^ false].
		lastElm := elm].
	^ true! !

!TSortable methodsFor: 'sorting' stamp: 'MarcusDenker 3/5/2010 16:48'!
isSortedBy: aBlock
	"Return true if the receiver is sorted by the given criterion."

	| lastElm elm |
	self isEmpty ifTrue: [^ true].
	lastElm := self first.
	2 to: self size do: 
		[:index | 
		elm := self at: index.
		(aBlock value: lastElm value: elm) ifFalse: [^ false].
		lastElm := elm].
	^ true! !

!TSortable methodsFor: 'sorting' stamp: 'MarcusDenker 3/5/2010 16:48'!
mergeFirst: first middle: middle last: last into: dst by: aBlock
	"Private. Merge the sorted ranges [first..middle] and [middle+1..last] 
	of the receiver into the range [first..last] of dst."

	| i1 i2 val1 val2 out |
	i1 := first.
	i2 := middle + 1.
	val1 := self at: i1.
	val2 := self at: i2.
	out := first - 1.  "will be pre-incremented"

	"select 'lower' half of the elements based on comparator"
	[(i1 <= middle) and: [i2 <= last]] whileTrue:
		[(aBlock value: val1 value: val2)
			ifTrue: [dst at: (out := out + 1) put: val1.
					val1 := self at: (i1 := i1 + 1)]
			ifFalse: [dst at: (out := out + 1) put: val2.
					i2 := i2 + 1.
					i2 <= last ifTrue: [val2 := self at: i2]]].

	"copy the remaining elements"
	i1 <= middle
		ifTrue: [dst replaceFrom: out + 1 to: last with: self startingAt: i1]
		ifFalse: [dst replaceFrom: out + 1 to: last with: self startingAt: i2]! !

!TSortable methodsFor: 'sorting' stamp: 'jannik.laval 5/1/2010 16:34'!
mergeSortFrom: startIndex to: stopIndex by: aBlock
	"Sort the given range of indices using the mergesort algorithm.
	Mergesort is a worst-case O(N log N) sorting algorithm that usually
	does only half as many comparisons as heapsort or quicksort."

	"Details: recursively split the range to be sorted into two halves,
	mergesort each half, then merge the two halves together. An extra 
	copy of the data is used as temporary storage and successive merge 
	phases copy data back and forth between the receiver and this copy.
	The recursion is set up so that the final merge is performed into the
	receiver, resulting in the receiver being completely sorted."

	self size <= 1 ifTrue: [^ self].  "nothing to do"
	startIndex = stopIndex ifTrue: [^ self].
	[startIndex >= 1 and: [startIndex < stopIndex]] assert. "bad start index"
	[stopIndex <= self size] assert. "bad stop index"
	self
		mergeSortFrom: startIndex
		to: stopIndex 
		src: self copy 
		dst: self 
		by: aBlock! !

!TSortable methodsFor: 'sorting' stamp: 'MarcusDenker 3/5/2010 16:49'!
mergeSortFrom: first to: last src: src dst: dst by: aBlock
	"Private. Split the range to be sorted in half, sort each half, and 
	merge the two half-ranges into dst."

	| middle |
	first = last ifTrue: [^ self].
	middle := (first + last) // 2.
	self mergeSortFrom: first to: middle src: dst dst: src by: aBlock.
	self mergeSortFrom: middle + 1 to: last src: dst dst: src by: aBlock.
	src mergeFirst: first middle: middle last: last into: dst by: aBlock! !

!TSortable methodsFor: 'sorting' stamp: 'MarcusDenker 3/5/2010 16:49'!
sort
	"Sort this collection into ascending order using the '<=' operator."

	self sort: [:a :b | a <= b]! !

!TSortable methodsFor: 'sorting' stamp: 'MarcusDenker 3/5/2010 16:49'!
sort: aSortBlock 
	"Sort this array using aSortBlock. The block should take two arguments
	and return true if the first element should preceed the second one."

	self
		mergeSortFrom: 1
		to: self size
		by: aSortBlock! !

Trait named: #TTokenReplace
	uses: {}
	category: 'Collections-Traits'!

!TTokenReplace methodsFor: 'copying' stamp: 'CamilloBruni 9/6/2011 09:58'!
copyReplaceAll: oldSubstring with: newSubstring asTokens: ifTokens
	^ ifTokens 
			ifTrue: [ self copyReplaceTokens: oldSubstring with: newSubstring ]
			ifFalse: [ self copyReplaceAll: oldSubstring with: newSubstring ]
! !

!TTokenReplace methodsFor: 'copying' stamp: 'CamilloBruni 9/6/2011 10:05'!
copyReplaceTokens: oldSubstring with: newSubstring
	"Answer a copy of the receiver in which all occurrences of oldSubstring have been replaced by newSubstring. All the characters surrounding the recplacement must not be alphanumeric."

	| startSearch currentIndexOf searchIncrement|
	startSearch := 1.
	searchIncrement := newSubstring size.
	^ self class new: self size streamContents: [ :stream|
		[(currentIndexOf := self indexOfSubCollection: oldSubstring startingAt: startSearch) > 0]
			whileTrue: [
				stream next: (currentIndexOf - startSearch) putAll: self startingAt: startSearch.
				
				((currentIndexOf = 1 or: [(self at: currentIndexOf-1) isAlphaNumeric not])
				 and: [currentIndexOf = self size or: [(self at: currentIndexOf+1) isAlphaNumeric not]])
					ifTrue: [
						stream nextPutAll: newSubstring.
						startSearch := currentIndexOf + searchIncrement]
					ifFalse: [ startSearch := currentIndexOf]].
		
		"copy over the last leftover pieces"
		stream next: (self size - startSearch + 1) putAll: self startingAt: startSearch].
! !
